/*
Copyright Frédéric LEGRAND, Lycée Chateaubriand, Rennes (2010) 

Ce logiciel est un programme informatique servant d'interface logiciel
pour le convertisseur analogique-numérique Eurosmart SysamPCI

Ce logiciel est régi par la licence CeCILL soumise au droit français et
respectant les principes de diffusion des logiciels libres. Vous pouvez
utiliser, modifier et/ou redistribuer ce programme sous les conditions
de la licence CeCILL telle que diffusée par le CEA, le CNRS et l'INRIA 
sur le site "http://www.cecill.info".

En contrepartie de l'accessibilité au code source et des droits de copie,
de modification et de redistribution accordés par cette licence, il n'est
offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
seule une responsabilité restreinte pèse sur l'auteur du programme,  le
titulaire des droits patrimoniaux et les concédants successifs.

A cet égard  l'attention de l'utilisateur est attirée sur les risques
associés au chargement,  à l'utilisation,  à la modification et/ou au
développement et à la reproduction du logiciel par l'utilisateur étant 
donné sa spécificité de logiciel libre, qui peut le rendre complexe à 
manipuler et qui le réserve donc à des développeurs et des professionnels
avertis possédant  des  connaissances  informatiques approfondies.  Les
utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
logiciel à leurs besoins dans des conditions permettant d'assurer la
sécurité de leurs systèmes et ou de leurs données et, plus généralement, 
à l'utiliser et l'exploiter dans les mêmes conditions de sécurité. 

Le fait que vous puissiez accéder à cet en-tête signifie que vous avez 
pris connaissance de la licence CeCILL, et que vous en avez accepté les
termes.
*/
#include <stdio.h>
#include "sysam.h"
#include "sysamPCILink.h"

HINSTANCE _HandleSysteme =NULL;
PtrSysteme systeme;

Type_SystemePresent* SystemePresent;
Type_RenvoyerSysteme* RenvoyerSysteme;
Type_LireInfos* LireInfos;
Type_OkEtatSysteme* OkEtatSysteme;
Type_ProgrammerEtatSysteme* ProgrammerEtatSysteme;
Type_EcrireTempo* EcrireTempo;
Type_LireTempo* LireTempo;
Type_FermetureDLL* FermetureDLL;



Type_LimitesCalibreEA* LimitesCalibreEA;
Type_LireCalibreEA* LireCalibreEA;
Type_EcrireCalibreEA* EcrireCalibreEA;
Type_LireGain* LireGain;
Type_EcrireGain* EcrireGain;
Type_LireEA_Entier* LireEA_Entier;
Type_LireEA_Reel* LireEA_Reel;
Type_BinVoltEA* BinVoltEA;
Type_VoltBinEA* VoltBinEA;

Type_LimitesCalibreSA* LimitesCalibreSA;
Type_LireCalibreSA* LireCalibreSA;
Type_EcrireCalibreSA* EcrireCalibreSA;
Type_EcrireSA_Entier* EcrireSA_Entier;
Type_EcrireSA_Reel* EcrireSA_Reel;
Type_BinVoltSA* BinVoltSA;
Type_VoltBinSA* VoltBinSA;
Type_LirePort* LirePort;
Type_EcrirePort* EcrirePort;

Type_LireBit* LireBit;
Type_EcrireBit* EcrireBit;
Type_LireSensPort* LireSensPort;
Type_ProgrammerSensPort* ProgrammerSensPort;

Type_PreparerAcquisition* PreparerAcquisition;
Type_AcquerirTable* AcquerirTable;
Type_ProgrammerDeclenchement* ProgrammerDeclenchement;
Type_PreparerEmission* PreparerEmission;
Type_EmettreSorties* EmettreSorties;
Type_ProgrammerAccesRapide* ProgrammerAccesRapide;


int nptmax[8] = {32700, 22400,14900,11200,8900,7400,6400,5600};

/*----------------------------------------------------------------------------------------------
        Ouverture de la DLL SysamPCI.DLL et chargement des pointeurs de fonctions
*/
int pci_ouvrir(Handle *handle) {
    int i;
    if (!(_HandleSysteme=LoadLibrary(TEXT("SYSAMPCI.dll")))) {
        sprintf_s(handle->errormsg,MSGSIZE,"%s","Erreur : SYSAMPCI.dll introuvable");
        return 0;
    }
	SystemePresent=(Type_SystemePresent*) GetProcAddress(_HandleSysteme,"SystemePresent");
    RenvoyerSysteme=(Type_RenvoyerSysteme*) GetProcAddress(_HandleSysteme,"RenvoyerSysteme");
    LireInfos=(Type_LireInfos*) GetProcAddress(_HandleSysteme,"LireInfos");
    OkEtatSysteme=(Type_OkEtatSysteme*) GetProcAddress(_HandleSysteme,"OkEtatSysteme");
    ProgrammerEtatSysteme=(Type_ProgrammerEtatSysteme*) GetProcAddress(_HandleSysteme,"ProgrammerEtatSysteme");
    EcrireTempo=(Type_EcrireTempo*) GetProcAddress(_HandleSysteme,"EcrireTempo");
    LireTempo=(Type_LireTempo*) GetProcAddress(_HandleSysteme,"LireTempo");
    FermetureDLL=(Type_FermetureDLL*) GetProcAddress(_HandleSysteme,"FermetureDLL");

    LimitesCalibreEA=(Type_LimitesCalibreEA*) GetProcAddress(_HandleSysteme,"LimitesCalibreEA");
    LireCalibreEA=(Type_LireCalibreEA*) GetProcAddress(_HandleSysteme,"LireCalibreEA");
    EcrireCalibreEA=(Type_EcrireCalibreEA*) GetProcAddress(_HandleSysteme,"EcrireCalibreEA");
    LireGain=(Type_LireGain*) GetProcAddress(_HandleSysteme,"LireGain");
    EcrireGain=(Type_EcrireGain*) GetProcAddress(_HandleSysteme,"EcrireGain");
    LireEA_Entier=(Type_LireEA_Entier*) GetProcAddress(_HandleSysteme,"LireEA_Entier");
    LireEA_Reel=(Type_LireEA_Reel*) GetProcAddress(_HandleSysteme,"LireEA_Reel");
    BinVoltEA=(Type_BinVoltEA*) GetProcAddress(_HandleSysteme,"BinVoltEA");
    VoltBinEA=(Type_VoltBinEA*) GetProcAddress(_HandleSysteme,"VoltBinEA");

    LimitesCalibreSA=(Type_LimitesCalibreSA*) GetProcAddress(_HandleSysteme,"LimitesCalibreSA");
    LireCalibreSA=(Type_LireCalibreSA*) GetProcAddress(_HandleSysteme,"LireCalibreSA");
    EcrireCalibreSA=(Type_EcrireCalibreSA*) GetProcAddress(_HandleSysteme,"EcrireCalibreSA");
    EcrireSA_Entier=(Type_EcrireSA_Entier*) GetProcAddress(_HandleSysteme,"EcrireSA_Entier");
    EcrireSA_Reel=(Type_EcrireSA_Reel*) GetProcAddress(_HandleSysteme,"EcrireSA_Reel");
    BinVoltSA=(Type_BinVoltSA*) GetProcAddress(_HandleSysteme,"BinVoltSA");
    VoltBinSA=(Type_VoltBinSA*) GetProcAddress(_HandleSysteme,"VoltBinSA");
    LirePort=(Type_LirePort*) GetProcAddress(_HandleSysteme,"LirePort");
    EcrirePort=(Type_EcrirePort*) GetProcAddress(_HandleSysteme,"EcrirePort");

    LireBit=(Type_LireBit*) GetProcAddress(_HandleSysteme,"LireBit");
    EcrireBit=(Type_EcrireBit*) GetProcAddress(_HandleSysteme,"EcrireBit");
    LireSensPort=(Type_LireSensPort*) GetProcAddress(_HandleSysteme,"LireSensPort");
    ProgrammerSensPort=(Type_ProgrammerSensPort*) GetProcAddress(_HandleSysteme,"ProgrammerSensPort");

    PreparerAcquisition=(Type_PreparerAcquisition*) GetProcAddress(_HandleSysteme,"PreparerAcquisition");
    AcquerirTable=(Type_AcquerirTable*) GetProcAddress(_HandleSysteme,"AcquerirTable");
    ProgrammerDeclenchement=(Type_ProgrammerDeclenchement*) GetProcAddress(_HandleSysteme,"ProgrammerDeclenchement");
    PreparerEmission=(Type_PreparerEmission*) GetProcAddress(_HandleSysteme,"PreparerEmission");
    EmettreSorties=(Type_EmettreSorties*) GetProcAddress(_HandleSysteme,"EmettreSorties");
    ProgrammerAccesRapide=(Type_ProgrammerAccesRapide*) GetProcAddress(_HandleSysteme,"ProgrammerAccesRapide");
    
    
    
    if (SystemePresent()) {
        systeme = RenvoyerSysteme(); // étape nécessaire (sinon plantage de la fonction EmettreSortie)
        handle->present = 1;
        handle->nombreEA = 0;
        for (i=0; i<8; i++) {
            handle->calibreEA[i] = 1;
            EcrireCalibreEA(i,1);
        }
        handle->nbcalibres = 5;
        handle->maxCalibreEA[0] = 0; handle->maxCalibreEA[1] = 10.0; handle->maxCalibreEA[2] = 5.0;   handle->maxCalibreEA[3] = 2.5; handle->maxCalibreEA[4]=1.0;
        return 1;
    }
    else {
        sprintf_s(handle->errormsg,MSGSIZE,"%s","Erreur : carte sysamPCI absente");
        return 0;
    }
}

/*----------------------------------------------------------------------------------------------
        Fermeture de la DLL
*/
int pci_fermer(Handle *handle) {
    FreeLibrary(_HandleSysteme);
    return 1;
}
/*-----------------------------------------------------------------------------------------------
        Configuration des entrées analogiques (convertisseur A/N)
            listeVoies : tableau des voies sélectionnées
            nbVoies : taille du tableau précédent
            listeCalibres : tableau des tensions maximales en volt pour les voies sélectionnées
            nbCalibres : taille du tableau précédent
            listeDiff : tableau des voies en mode différentiel
            nbDiff : taille du tableau précédent
*/
int pci_config_entrees(Handle *handle,int *listeVoies, int nbVoies, double *listeCalibres, int nbCalibres, int *listeDiff, int nbDiff){
    int v,i,j;
    if (nbDiff>0) {
        ProgrammerEtatSysteme(ca_ModeDiff,1);
    }
    else {
        ProgrammerEtatSysteme(ca_ModeDiff,0);
    }
    v = 0;
    for (i=0; i < nbVoies; i++) {
        if (((listeVoies[i]==4)||(listeVoies[i]==5)||(listeVoies[i]==6)||(listeVoies[i]==7))&(nbDiff>0)) {
            // voie secondaire en mode différentiel, pas prise en compe
        }
        else {
            handle->voiesEA[v] = listeVoies[i];
            v += 1;
        }
    }
    handle->nombreEA = v; // nombre de tensions à lire (max 4 en mode différentiel)
    for (i=0; i<nbCalibres; i++) { // numéros des calibres sélectionnés selon la tension maximale
        handle->calibreEA[i] = 1;
        if (listeCalibres[i] > 10.0) {
            sprintf_s(handle->errormsg,MSGSIZE,"%s","Erreur de calibre : tension max trop grande");
            return 0;
        }
        for (j=0; j < handle->nbcalibres; j++) {
            if (handle->maxCalibreEA[j] > listeCalibres[i]) handle->calibreEA[i] = j; 
        }
    }
    for (i=0; i < handle->nombreEA; i++) {
        if (handle->voiesEA[i]<8) {
            EcrireCalibreEA((BYTE)(handle->voiesEA[i]),(BYTE)(handle->calibreEA[i]));
        }
    }
    if (handle->nombreEA==1) handle->temin = 0.5;
    else handle->temin = 2.5*handle->nombreEA;
    return 1;
}

/*---------------------------------------------------------------------------------
        Configuration du trigger
            externe : 1 pour un déclenchement sur la voie EXT, 0 pour un déclenchement sur voie
            voie : voie de déclenchement ou -1 pour désactiver le trigger
            seuil : seuil du déclenchement
            montant : 1 pour un déclenchement sur front montant, 0 pour un front descendant
            pretrigger : non utilisé
            pretriggerSouple : non utilisé
            hysteresis : non utilisé
            
*/
int pci_config_trigger(Handle *handle,int externe, int voie, double seuil, int montant, int pretrigger, int pretriggerSouple, int hysteresis){
    int m;
    m = (~montant)&1;
    if (externe) {
        ProgrammerDeclenchement(CodeSynchroExterne,seuil,m);
        return 1;
    }
    ProgrammerDeclenchement(voie,seuil,m);
    return 1;
}


/*--------------------------------------------------------------------------------------------------------------------
        Configuration de l'échantillonnage du convertisseur analogique->numérique
            temps : période d'échantillonnage en microsecondes
            nbpoints : nombre de points à acquérir
*/
int pci_config_echantillon(Handle *handle, double temps, int nbpoints) {
    handle->nbpointsEA = nbpoints;
    handle->nbpointsEAdemande = nbpoints;
    if (temps >= handle->temin) {
        handle->tempsEchantillon = temps;
    }
    else handle->tempsEchantillon = handle->temin;
    return 1;
}

/*-------------------------------------------------------------------------------------------------
        Acquisition complète et mémorisation du temps et des tensions lues
        La fonction retourne à la fin de l'acquisition

*/
int pci_acquerir(Handle *handle) {
    short *data;
    int nbpoints;
    int i,j,k;
    double shift = 0;
    TTableVoie tableVoies;
    int v;
    if (handle->present==0) {
        sprintf_s(handle->errormsg,MSGSIZE,"%s","Erreur : CAN non ouvert");
        return 0;
    }
    // max 5621 pour 8 voies
    // max 6425 pour 7 voies
    // max 7496 pour 6 voies
    // max 8993 pour 5 voies simples
    // max 11243 pour 4 voies simples ou differentielles
    // max 14990 pour 3 voies simples ou differentielles
    // max 22484 points pour 2 voies simples ou differentielles
    // max 32754 points pour 1 voie simples ou differentielles
    if (handle->nbpointsEA > nptmax[handle->nombreEA-1]) {
    	sprintf_s(handle->errormsg,MSGSIZE,"Erreur : nombre de points trop grand (max %d)",nptmax[handle->nombreEA-1]);
        return 0;   
    }
    for (v=0; v < handle->nombreEA; v++) {
        tableVoies[v] = (__int8) (handle->voiesEA[v]);
    }
    tableVoies[v] = -1;
    PreparerAcquisition(handle->nbpointsEA,1,handle->tempsEchantillon,&tableVoies,NULL);
    
    data = malloc(sizeof(short)*handle->nombreEA*(handle->nbpointsEA+1));
    handle->nbpointsEAlus = 0;
    AcquerirTable(data,&nbpoints);
    handle->nbpointsEA = nbpoints;
    if (handle->tabEntrees!=NULL) free(handle->tabEntrees);
    if (handle->tabTemps!=NULL) free(handle->tabTemps);
    handle->tabEntrees = malloc(sizeof(double)*nbpoints*handle->nombreEA);
    handle->tabTemps = malloc(sizeof(double)*nbpoints*handle->nombreEA);
    if (handle->tempsEchantillon < handle->temin) handle->tempsEchantillon = handle->temin;
    if (handle->nombreEA > 1) shift = 2.5e-6;
    k = 0;
    for (i=0; i < nbpoints; i++) {
        for (j=0; j < handle->nombreEA; j++) {
            handle->tabEntrees[k+j] = ((double)data[k+j+1]-2048)/2048*handle->maxCalibreEA[handle->calibreEA[handle->voiesEA[j]]]; 
            handle->tabTemps[k+j] = ((double)i)*handle->tempsEchantillon*1e-6+shift*j;
        }
        k += handle->nombreEA;
    }
    free(data);
    handle->nbpointsEAlus = nbpoints;
    return 1;
}

int pci_lancer(Handle *handle) {
   sprintf_s(handle->errormsg,MSGSIZE,"%s","Erreur : acquisition parallele impossible sur SysamPCI");
   return 0;
}
int pci_lancer_avec_sorties(Handle *handle,double *valeurs1, int nbvaleurs1,double *valeurs2, int nbvaleurs2) {
   sprintf_s(handle->errormsg,MSGSIZE,"%s","Erreur : acquisition parallele impossible sur SysamPCI");
   return 0;
}


/*--------------------------------------------------------------------------------------------------------------
        Acquisition avec utilisation synchrone des sorties
            valeurs1 : tableau des tensions à appliquer sur la sortie 1
            nbvaleurs1 : longueur du tableau précédent
            valeurs2 : non utilisé
            nbvaleurs2 : non utilisé

*/
int pci_acquerir_avec_sorties(Handle *handle, double *valeurs1, int nbvaleurs1,double *valeurs2, int nbvaleurs2) {
    short *data;
    int nbpoints;
    int i,j,k,m;
    double shift = 0; 
    TTableVoie tableVoies;
    int v;
    if (handle->present==0) {
        sprintf_s(handle->errormsg,MSGSIZE,"%s","Erreur : CAN non ouvert");
        return 0;
    }
    if (handle->nbpointsEA != nbvaleurs1) {
        sprintf_s(handle->errormsg,MSGSIZE,"%s","Erreur : nombre de points incorrect");
        return 0;
    }
    if (nbvaleurs2!=0) {
        sprintf_s(handle->errormsg,MSGSIZE,"%s","Erreur : une seule sortie sur SysamPCI");
        return 0;
    }
    // max 5618 pour 8 voies
    // max 6422 pour 7 voies
    // max 7493 pour 6 voies
    // max 8993 pour 5 voies
    // max 11243 points pour 4 voies simples ou differentielles
    // max 14993 points pour 3 voies simples ou differentielles
    // max 22493 points pour 2 voies simples ou differentielles
    // max 32761 points pour 1 voies simples ou differentielles
    if (handle->nbpointsEA > nptmax[handle->nombreEA-1]) {
    	sprintf_s(handle->errormsg,MSGSIZE,"Erreur : nombre de points trop grand (max %d)",nptmax[handle->nombreEA-1]);
        return 0;   
    }
    for (v=0; v < handle->nombreEA; v++) {
        tableVoies[v] = (unsigned char) (handle->voiesEA[v]);
    }
    tableVoies[v++] = 9; // sélection de la sortie
    tableVoies[v] = -1;
    PreparerAcquisition(handle->nbpointsEA,1,handle->tempsEchantillon,&tableVoies,NULL);
    data = malloc(sizeof(short)*(handle->nombreEA+1)*(handle->nbpointsEA+1));
    for (k=1; k<=handle->nbpointsEA; k++) {
        data[k*(handle->nombreEA+1)] = (short) ((valeurs1[k]+10.0)*204.75);
    }
    AcquerirTable(data,&nbpoints);
    handle->nbpointsEA = nbpoints;
    if (handle->tabEntrees!=NULL) free(handle->tabEntrees);
    if (handle->tabTemps!=NULL) free(handle->tabTemps);
    handle->tabEntrees = malloc(sizeof(double)*nbpoints*handle->nombreEA);
    handle->tabTemps = malloc(sizeof(double)*nbpoints*handle->nombreEA);
    if (handle->tempsEchantillon < handle->temin) handle->tempsEchantillon = handle->temin;
    if (handle->nombreEA > 1) shift = 2.5e-6;
    k = 0;
    m = 0;
    for (i=0; i < nbpoints; i++) {
        for (j=0; j < handle->nombreEA; j++) {
            handle->tabEntrees[m+j] = ((double)data[k+j+1]-2048)/2048*handle->maxCalibreEA[handle->calibreEA[handle->voiesEA[j]]]; 
            handle->tabTemps[m+j] = ((double)i)*handle->tempsEchantillon*1e-6+shift*j;
        }
        m += handle->nombreEA;
        k += handle->nombreEA+1;
    }
    free(data);
    handle->nbpointsEAlus = nbpoints;
    return 1;
}
/*----------------------------------------------------------------------------------------------------------------------------
        Configuration d'une sortie
            nsortie : numéro de la sortie (1 ou 2)
            te : période d'échantillonnage en microsecondes
            valeurs : tableau des tensions à appliquer en volt
            npoints : longueur du tableau précédent
            repetition : 1 pour obtenir une répétition périodique, 0 sinon
*/
int pci_config_sortie(Handle *handle, int nsortie, double te, double *valeurs, int nbvaleurs, int repetition) {
    int i;
    TTableVoie tv;
    if (nsortie!=1) {
        sprintf_s(handle->errormsg,MSGSIZE,"%s","Erreur : numero de sortie incorrect");
        return 0;
    }
    if (te < 1.0) {
        sprintf_s(handle->errormsg,MSGSIZE,"%s","Erreur : temps d'echantillonage de sortie trop faible (min 1 microseconde)");
        return 0;
    }
    handle->tempsEchantillonSorties = te;
    handle->repetition = repetition;
    if (handle->tabSortieS1!=NULL) free(handle->tabSortieS1);
    handle->nbpointsS1 = nbvaleurs;
    handle->tabSortieS1 = malloc(sizeof(short)*(nbvaleurs+1));
    handle->tabSortieS1[0] = 0;
    for (i=0; i< nbvaleurs; i++) {
        handle->tabSortieS1[i+1] = (short)((valeurs[i]+10.0)*204.75);
    }
    tv[0] = NumSortie1; tv[1]=-1;
    PreparerEmission(handle->nbpointsS1,handle->repetition,handle->tempsEchantillonSorties,&tv,NULL);
    return 1;
}

/*----------------------------------------------------------------------------------------------------------
    Déclencher les sorties
        ns1 : 1 pour déclencher la sortie 1, 0 sinon
        ns2 : non utilisé
*/

int pci_declencher_sorties(Handle *handle, int ns1, int ns2) {
    if (ns2!=0) {
        sprintf_s(handle->errormsg,MSGSIZE,"%s","Erreur : une seule sortie sur Sysam PCI");
        return 0;
    }
    if (ns1==1) {
        EmettreSorties(handle->tabSortieS1); 
    }
    return 1;
}
/*------------------------------------------------------------------------------------------------------------
    Stopper les sorties :
        ns1 : 1 pour stopper la sortie 1, 0 sinon
        ns2 : non utilisé

*/
int pci_stopper_sorties(Handle *handle, int ns1, int ns2) {
    if (ns2!=0) {
        sprintf_s(handle->errormsg,MSGSIZE,"%s","Erreur : une seule sortie sur Sysam PCI");
        return 0;
    }
    if (ns1==1) {
        EcrireSA_Entier(1,2048);
    }
    return 1;
}
/*-----------------------------------------------------------------------------------------------------------------
        Activer la lecture directe (sans échantillonnage) sur les entrées
            listeVoies : tableau des voies
            nbvoies : taille du tableau précédent
*/
int pci_activer_lecture(Handle *handle, int* listeVoies, int nbVoies, int latch) {
    int v;
    handle->nombreEAdirect = nbVoies;
    for (v=0; v < nbVoies; v++) {
        ProgrammerAccesRapide(TRUE,listeVoies[v],1);
        handle->voiesEAdirect[v] = listeVoies[v];
    }
    return 1;
}
/*------------------------------------------------------------------------------------------------
    Effectuer une lecture directe des entrées sélectionnées
*/
int pci_lire(Handle *handle) {
    int valeur[8];
    int v;
    for (v=0; v < handle->nombreEAdirect; v++) {
        valeur[v] = LireEA_Entier(handle->voiesEAdirect[v],1);
    }
    for (v=0; v < handle->nombreEAdirect; v++) {
        handle->tensionsLues[v] = ((double)valeur[v]-2048)/2048* handle->maxCalibreEA[handle->calibreEA[handle->voiesEAdirect[v]]];
    }
    return 1;
}
/*-------------------------------------------------------------------------------------------------------
        Désactiver la lecture directe
*/
int pci_desactiver_lecture(Handle *handle){
    int v;
    for (v=0; v < 8; v++) {
        ProgrammerAccesRapide(FALSE,v,1);
    }
    return 1;
}
/*--------------------------------------------------------------------------------------------------------
        Ecrire une tension sur les sorties
            ns1 : 1 pour écrire la sortie 1, 0 sinon
            value1 : tension en volt à appliquer sur la sortie 1
            ns2 : non utilisé
            value2 : non utilisé
*/
int pci_ecrire(Handle *handle, int ns1, double value1, int ns2, double value2) {
    if (ns2!=0) {
        sprintf_s(handle->errormsg,MSGSIZE,"%s","Erreur : une seule sortie sur Sysam PCI");
        return 0;
    }
    if (ns1==1) {
        EcrireSA_Entier(1,(short) floor((value1+10.0)*204.75)); 
    }
    return 1;
}
